@@ -20,12 +20,14 @@
 
 #include <config.h>
 
+#include <fcntl.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <curl/curl.h>
 
 #include "datatypes.h"
 #include "ch_conf.h"
+#include "ch_events.h"
 #include "ch_interface.h"
 #include "ch_monitor.h"
 #include "domain_interface.h"
@@ -56,7 +58,8 @@ static int virCHMonitorOnceInit(void)
 VIR_ONCE_GLOBAL_INIT(virCHMonitor);
 
 int virCHMonitorShutdownVMM(virCHMonitor *mon);
-int virCHMonitorPutNoContent(virCHMonitor *mon, const char *endpoint);
+int virCHMonitorPutNoContent(virCHMonitor *mon, const char *endpoint,
+                             domainLogContext *logCtxt);
 
 static int
 virCHMonitorBuildCPUJson(virJSONValue *content, virDomainDef *vmdef)
@@ -89,21 +92,34 @@ virCHMonitorBuildCPUJson(virJSONValue *content, virDomainDef *vmdef)
 }
 
 static int
-virCHMonitorBuildPTYJson(virJSONValue *content, virDomainDef *vmdef)
+virCHMonitorBuildConsoleJson(virJSONValue *content,
+                             virDomainDef *vmdef)
 {
-    if (vmdef->nconsoles) {
-        g_autoptr(virJSONValue) pty = virJSONValueNewObject();
-        if (virJSONValueObjectAppendString(pty, "mode", "Pty") < 0)
+    g_autoptr(virJSONValue) console = virJSONValueNewObject();
+    g_autoptr(virJSONValue) serial = virJSONValueNewObject();
+
+    if (vmdef->nconsoles &&
+        vmdef->consoles[0]->source->type == VIR_DOMAIN_CHR_TYPE_PTY) {
+        if (virJSONValueObjectAppendString(console, "mode", "Pty") < 0)
             return -1;
-        if (virJSONValueObjectAppend(content, "console", &pty) < 0)
+        if (virJSONValueObjectAppend(content, "console", &console) < 0)
             return -1;
     }
 
     if (vmdef->nserials) {
-        g_autoptr(virJSONValue) pty = virJSONValueNewObject();
-        if (virJSONValueObjectAppendString(pty, "mode", "Pty") < 0)
-            return -1;
-        if (virJSONValueObjectAppend(content, "serial", &pty) < 0)
+        if (vmdef->serials[0]->source->type == VIR_DOMAIN_CHR_TYPE_PTY) {
+            if (virJSONValueObjectAppendString(serial, "mode", "Pty") < 0)
+                return -1;
+        } else if (vmdef->serials[0]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX) {
+            if (virJSONValueObjectAppendString(serial, "mode", "Socket") < 0)
+                return -1;
+            if (virJSONValueObjectAppendString(serial,
+                                               "socket",
+                                               vmdef->serials[0]->source->data.file.path) < 0)
+                return -1;
+        }
+
+        if (virJSONValueObjectAppend(content, "serial", &serial) < 0)
             return -1;
     }
 
@@ -265,18 +281,25 @@ virCHMonitorBuildDisksJson(virJSONValue *content, virDomainDef *vmdef)
 /**
  * virCHMonitorBuildNetJson:
  * @net: pointer to a guest network definition
+ * @netindex: index of the guest network definition
  * @jsonstr: returned network json
  *
  * Build net json to send to CH
  * Returns 0 on success or -1 in case of error
  */
 int
-virCHMonitorBuildNetJson(virDomainNetDef *net, char **jsonstr)
+virCHMonitorBuildNetJson(virDomainNetDef *net,
+                         int netindex,
+                         char **jsonstr)
 {
     char macaddr[VIR_MAC_STRING_BUFLEN];
     g_autoptr(virJSONValue) net_json = virJSONValueNewObject();
     virDomainNetType actualType = virDomainNetGetActualType(net);
 
+    g_autofree char *id = g_strdup_printf("%s_%d", CH_NET_ID_PREFIX, netindex);
+    if (virJSONValueObjectAppendString(net_json, "id", id) < 0)
+        return -1;
+
     if (actualType == VIR_DOMAIN_NET_TYPE_ETHERNET &&
         net->guestIP.nips == 1) {
         const virNetDevIPAddr *ip;
@@ -415,7 +438,7 @@ virCHMonitorBuildVMJson(virCHDriver *driver, virDomainDef *vmdef,
         return -1;
     }
 
-    if (virCHMonitorBuildPTYJson(content, vmdef) < 0)
+    if (virCHMonitorBuildConsoleJson(content, vmdef) < 0)
         return -1;
 
     if (virCHMonitorBuildCPUJson(content, vmdef) < 0)
@@ -444,6 +467,22 @@ virCHMonitorBuildVMJson(virCHDriver *driver, virDomainDef *vmdef,
     return 0;
 }
 
+static int
+virCHMonitorBuildKeyValueStringJson(char **jsonstr,
+                                    const char *key,
+                                    const char *value)
+{
+    g_autoptr(virJSONValue) content = virJSONValueNewObject();
+
+    if (virJSONValueObjectAppendString(content, key, value) < 0)
+        return -1;
+
+    if (!(*jsonstr = virJSONValueToString(content, false)))
+        return -1;
+
+    return 0;
+}
+
 static int
 chMonitorCreateSocket(const char *socket_path)
 {
@@ -500,11 +539,12 @@ chMonitorCreateSocket(const char *socket_path)
 }
 
 virCHMonitor *
-virCHMonitorNew(virDomainObj *vm, const char *socketdir)
+virCHMonitorNew(virDomainObj *vm, virCHDriverConfig *cfg, int logfile)
 {
     g_autoptr(virCHMonitor) mon = NULL;
     g_autoptr(virCommand) cmd = NULL;
     int socket_fd = 0;
+    int event_monitor_fd;
 
     if (virCHMonitorInitialize() < 0)
         return NULL;
@@ -512,6 +552,9 @@ virCHMonitorNew(virDomainObj *vm, const char *socketdir)
     if (!(mon = virObjectLockableNew(virCHMonitorClass)))
         return NULL;
 
+    /* avoid VIR_FORCE_CLOSE()-ing garbage fd value in virCHMonitorClose */
+    mon->eventmonitorfd = -1;
+
     if (!vm->def) {
         virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                        _("VM is not defined"));
@@ -519,15 +562,46 @@ virCHMonitorNew(virDomainObj *vm, const char *socketdir)
     }
 
     /* prepare to launch Cloud-Hypervisor socket */
-    mon->socketpath = g_strdup_printf("%s/%s-socket", socketdir, vm->def->name);
-    if (g_mkdir_with_parents(socketdir, 0777) < 0) {
+    mon->socketpath = g_strdup_printf("%s/%s-socket", cfg->stateDir, vm->def->name);
+    if (g_mkdir_with_parents(cfg->stateDir, 0777) < 0) {
         virReportSystemError(errno,
                              _("Cannot create socket directory '%1$s'"),
-                             socketdir);
+                             cfg->stateDir);
+        return NULL;
+    }
+
+    if (g_mkdir_with_parents(cfg->saveDir, 0777) < 0) {
+        virReportSystemError(errno,
+                             _("Cannot create save directory '%1$s'"),
+                             cfg->saveDir);
+        return NULL;
+    }
+
+    /* Event monitor file to listen for VM state changes */
+    mon->eventmonitorpath = g_strdup_printf("%s/%s-event-monitor-fifo",
+                                            cfg->stateDir, vm->def->name);
+    if (virFileExists(mon->eventmonitorpath)) {
+        VIR_WARN("Monitor file (%s) already exists, trying to delete!",
+                  mon->eventmonitorpath);
+        if (virFileRemove(mon->eventmonitorpath, -1, -1) < 0) {
+            virReportError(VIR_ERR_INTERNAL_ERROR,
+                           _("Failed to remove the file: %1$s"),
+                           mon->eventmonitorpath);
+            return NULL;
+        }
+    }
+
+    if (mkfifo(mon->eventmonitorpath, S_IWUSR | S_IRUSR) < 0 &&
+            errno != EEXIST) {
+        virReportSystemError(errno, "%s",
+                             _("Cannot create monitor FIFO"));
         return NULL;
     }
 
     cmd = virCommandNew(vm->def->emulator);
+    virCommandSetOutputFD(cmd, &logfile);
+    virCommandSetErrorFD(cmd, &logfile);
+    virCommandNonblockingFDs(cmd);
     virCommandSetUmask(cmd, 0x002);
     socket_fd = chMonitorCreateSocket(mon->socketpath);
     if (socket_fd < 0) {
@@ -541,16 +615,42 @@ virCHMonitorNew(virDomainObj *vm, const char *socketdir)
     virCommandAddArgFormat(cmd, "fd=%d", socket_fd);
     virCommandPassFD(cmd, socket_fd, VIR_COMMAND_PASS_FD_CLOSE_PARENT);
 
+    virCommandAddArg(cmd, "--event-monitor");
+    virCommandAddArgFormat(cmd, "path=%s", mon->eventmonitorpath);
+
     /* launch Cloud-Hypervisor socket */
     if (virCommandRunAsync(cmd, &mon->pid) < 0)
         return NULL;
 
-    /* get a curl handle */
-    mon->handle = curl_easy_init();
+    /* open the reader end of fifo before start Event Handler */
+    while ((event_monitor_fd = open(mon->eventmonitorpath, O_RDONLY)) < 0) {
+        if (errno == EINTR) {
+            /* 100 milli seconds */
+            g_usleep(100000);
+            continue;
+        }
+        /* Any other error should be a BUG(kernel/libc/libvirtd)
+         * (ENOMEM can happen on exceeding per-user limits)
+         */
+        VIR_ERROR(_("%1$s: Failed to open the event monitor FIFO(%2$s) read end!"),
+                  vm->def->name, mon->eventmonitorpath);
+        /* CH process(Writer) is blocked at this point as EventHandler(Reader)
+         * fails to open the FIFO.
+         */
+        return NULL;
+    }
+    mon->eventmonitorfd = event_monitor_fd;
+    VIR_DEBUG("%s: Opened the event monitor FIFO(%s)", vm->def->name, mon->eventmonitorpath);
 
     /* now has its own reference */
     mon->vm = virObjectRef(vm);
 
+    if (virCHStartEventHandler(mon) < 0)
+        return NULL;
+
+    /* get a curl handle */
+    mon->handle = curl_easy_init();
+
     return g_steal_pointer(&mon);
 }
 
@@ -578,11 +678,24 @@ void virCHMonitorClose(virCHMonitor *mon)
         curl_easy_cleanup(mon->handle);
 
     if (mon->socketpath) {
-        if (virFileRemove(mon->socketpath, -1, -1) < 0) {
-            VIR_WARN("Unable to remove CH socket file '%s'",
-                     mon->socketpath);
+        if (virFileRemove(mon->socketpath, -1, -1) < 0 &&
+            errno != ENOENT) {
+            VIR_WARN("Unable to remove CH socket file '%s': %s",
+                     mon->socketpath, g_strerror(errno));
         }
-        g_free(mon->socketpath);
+        g_clear_pointer(&mon->socketpath, g_free);
+    }
+
+    virCHStopEventHandler(mon);
+    if (mon->eventmonitorfd >= 0) {
+        VIR_FORCE_CLOSE(mon->eventmonitorfd);
+    }
+    if (mon->eventmonitorpath) {
+        if (virFileRemove(mon->eventmonitorpath, -1, -1) < 0) {
+            VIR_WARN("Unable to remove CH event monitor file '%s'",
+                     mon->eventmonitorpath);
+        }
+        g_clear_pointer(&mon->eventmonitorpath, g_free);
     }
 
     virObjectUnref(mon);
@@ -623,32 +736,6 @@ virCHMonitorCurlPerform(CURL *handle)
     return responseCode;
 }
 
-int
-virCHMonitorPutNoContent(virCHMonitor *mon, const char *endpoint)
-{
-    VIR_LOCK_GUARD lock = virObjectLockGuard(mon);
-    g_autofree char *url = NULL;
-    int responseCode = 0;
-    int ret = -1;
-
-    url = g_strdup_printf("%s/%s", URL_ROOT, endpoint);
-
-    /* reset all options of a libcurl session handle at first */
-    curl_easy_reset(mon->handle);
-
-    curl_easy_setopt(mon->handle, CURLOPT_UNIX_SOCKET_PATH, mon->socketpath);
-    curl_easy_setopt(mon->handle, CURLOPT_URL, url);
-    curl_easy_setopt(mon->handle, CURLOPT_UPLOAD, 1L);
-    curl_easy_setopt(mon->handle, CURLOPT_HTTPHEADER, NULL);
-
-    responseCode = virCHMonitorCurlPerform(mon->handle);
-
-    if (responseCode == 200 || responseCode == 204)
-        ret = 0;
-
-    return ret;
-}
-
 struct curl_data {
     char *content;
     size_t size;
@@ -671,6 +758,49 @@ curl_callback(void *contents, size_t size, size_t nmemb, void *userp)
     return content_size;
 }
 
+int
+virCHMonitorPutNoContent(virCHMonitor *mon, const char *endpoint,
+                         domainLogContext *logCtxt)
+{
+    VIR_LOCK_GUARD lock = virObjectLockGuard(mon);
+    g_autofree char *url = NULL;
+    int responseCode = 0;
+    int ret = -1;
+    struct curl_data data = {0};
+    struct curl_slist *headers = NULL;
+
+    url = g_strdup_printf("%s/%s", URL_ROOT, endpoint);
+
+    /* reset all options of a libcurl session handle at first */
+    curl_easy_reset(mon->handle);
+
+    curl_easy_setopt(mon->handle, CURLOPT_UNIX_SOCKET_PATH, mon->socketpath);
+    curl_easy_setopt(mon->handle, CURLOPT_URL, url);
+    curl_easy_setopt(mon->handle, CURLOPT_UPLOAD, 1L);
+    curl_easy_setopt(mon->handle, CURLOPT_HTTPHEADER, NULL);
+    curl_easy_setopt(mon->handle, CURLOPT_INFILESIZE, 0L);
+
+    headers = curl_slist_append(headers, "Accept: application/json");
+    curl_easy_setopt(mon->handle, CURLOPT_HTTPHEADER, headers);
+    curl_easy_setopt(mon->handle, CURLOPT_WRITEFUNCTION, curl_callback);
+    curl_easy_setopt(mon->handle, CURLOPT_WRITEDATA, (void *)&data);
+
+    responseCode = virCHMonitorCurlPerform(mon->handle);
+
+    if (logCtxt && data.size) {
+        /* Do this to append a NULL char at the end of data */
+        data.content = g_realloc(data.content, data.size + 1);
+        data.content[data.size] = 0;
+        domainLogContextWrite(logCtxt, "HTTP response code from CH: %d\n", responseCode);
+        domainLogContextWrite(logCtxt, "Response = %s\n", data.content);
+    }
+
+    if (responseCode == 200 || responseCode == 204)
+        ret = 0;
+
+    return ret;
+}
+
 static int
 virCHMonitorGet(virCHMonitor *mon, const char *endpoint, virJSONValue **response)
 {
@@ -814,7 +944,7 @@ virCHMonitorGetThreadInfo(virCHMonitor *mon,
 int
 virCHMonitorShutdownVMM(virCHMonitor *mon)
 {
-    return virCHMonitorPutNoContent(mon, URL_VMM_SHUTDOWN);
+    return virCHMonitorPutNoContent(mon, URL_VMM_SHUTDOWN, NULL);
 }
 
 int
@@ -854,33 +984,121 @@ virCHMonitorCreateVM(virCHDriver *driver, virCHMonitor *mon)
 }
 
 int
-virCHMonitorBootVM(virCHMonitor *mon)
+virCHMonitorBootVM(virCHMonitor *mon, domainLogContext *logCtxt)
 {
-    return virCHMonitorPutNoContent(mon, URL_VM_BOOT);
+    return virCHMonitorPutNoContent(mon, URL_VM_BOOT, logCtxt);
 }
 
 int
 virCHMonitorShutdownVM(virCHMonitor *mon)
 {
-    return virCHMonitorPutNoContent(mon, URL_VM_SHUTDOWN);
+    return virCHMonitorPutNoContent(mon, URL_VM_SHUTDOWN, NULL);
 }
 
 int
 virCHMonitorRebootVM(virCHMonitor *mon)
 {
-    return virCHMonitorPutNoContent(mon, URL_VM_REBOOT);
+    return virCHMonitorPutNoContent(mon, URL_VM_REBOOT, NULL);
 }
 
 int
 virCHMonitorSuspendVM(virCHMonitor *mon)
 {
-    return virCHMonitorPutNoContent(mon, URL_VM_Suspend);
+    return virCHMonitorPutNoContent(mon, URL_VM_Suspend, NULL);
 }
 
 int
 virCHMonitorResumeVM(virCHMonitor *mon)
 {
-    return virCHMonitorPutNoContent(mon, URL_VM_RESUME);
+    return virCHMonitorPutNoContent(mon, URL_VM_RESUME, NULL);
+}
+
+int
+virCHMonitorSaveVM(virCHMonitor *mon,
+                   const char *to)
+{
+    g_autofree char *url = NULL;
+    int responseCode = 0;
+    int ret = -1;
+    g_autofree char *payload = NULL;
+    g_autofree char *path_url = NULL;
+    struct curl_slist *headers = NULL;
+    struct curl_data data = {0};
+
+    url = g_strdup_printf("%s/%s", URL_ROOT, URL_VM_SAVE);
+
+    headers = curl_slist_append(headers, "Accept: application/json");
+    headers = curl_slist_append(headers, "Content-Type: application/json");
+
+    path_url = g_strdup_printf("file://%s", to);
+    if (virCHMonitorBuildKeyValueStringJson(&payload, "destination_url", path_url) != 0)
+        return -1;
+
+
+    VIR_WITH_OBJECT_LOCK_GUARD(mon) {
+        /* reset all options of a libcurl session handle at first */
+        curl_easy_reset(mon->handle);
+
+        curl_easy_setopt(mon->handle, CURLOPT_UNIX_SOCKET_PATH, mon->socketpath);
+        curl_easy_setopt(mon->handle, CURLOPT_URL, url);
+        curl_easy_setopt(mon->handle, CURLOPT_CUSTOMREQUEST, "PUT");
+        curl_easy_setopt(mon->handle, CURLOPT_HTTPHEADER, headers);
+        curl_easy_setopt(mon->handle, CURLOPT_POSTFIELDS, payload);
+        curl_easy_setopt(mon->handle, CURLOPT_WRITEFUNCTION, curl_callback);
+        curl_easy_setopt(mon->handle, CURLOPT_WRITEDATA, (void *)&data);
+
+        responseCode = virCHMonitorCurlPerform(mon->handle);
+    }
+
+    if (responseCode == 200 || responseCode == 204) {
+        ret = 0;
+    } else {
+        data.content = g_realloc(data.content, data.size + 1);
+        data.content[data.size] = 0;
+        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                       data.content);
+        g_free(data.content);
+    }
+
+    /* reset the libcurl handle to avoid leaking a stack pointer to data */
+    curl_easy_reset(mon->handle);
+    curl_slist_free_all(headers);
+    return ret;
+}
+
+int
+virCHMonitorBuildRestoreJson(virDomainDef *vmdef,
+                             const char *from,
+                             char **jsonstr)
+{
+    size_t i;
+    g_autoptr(virJSONValue) restore_json = virJSONValueNewObject();
+    g_autofree char *path_url = g_strdup_printf("file://%s", from);
+
+    if (virJSONValueObjectAppendString(restore_json, "source_url", path_url))
+        return -1;
+
+    /* Pass the netconfig needed to restore with new netfds */
+    if (vmdef->nnets) {
+        g_autoptr(virJSONValue) nets = virJSONValueNewArray();
+        for (i = 0; i < vmdef->nnets; i++) {
+            g_autoptr(virJSONValue) net_json = virJSONValueNewObject();
+            g_autofree char *id = g_strdup_printf("%s_%ld", CH_NET_ID_PREFIX, i);
+            if (virJSONValueObjectAppendString(net_json, "id", id) < 0)
+                return -1;
+            if (virJSONValueObjectAppendNumberInt(net_json, "num_fds", vmdef->nets[i]->driver.virtio.queues))
+                return -1;
+            if (virJSONValueArrayAppend(nets, &net_json) < 0)
+                return -1;
+        }
+        if (virJSONValueObjectAppend(restore_json, "net_fds", &nets))
+            return -1;
+    }
+
+    if (!(*jsonstr = virJSONValueToString(restore_json, false)))
+        return -1;
+
+    return 0;
 }
 
 /**
