@@ -975,14 +975,14 @@ cmdDaemonLogFilters(vshControl *ctl, const vshCmd *cmd)
         const char *filters = NULL;
         if ((vshCommandOptString(ctl, cmd, "filters", &filters) < 0 ||
              virAdmConnectSetLoggingFilters(priv->conn, filters, 0) < 0)) {
-            vshError(ctl, _("Unable to change daemon logging settings"));
+            vshError(ctl, "%s", _("Unable to change daemon logging settings"));
             return false;
         }
     } else {
         g_autofree char *filters = NULL;
         if (virAdmConnectGetLoggingFilters(priv->conn,
                                            &filters, 0) < 0) {
-            vshError(ctl, _("Unable to get daemon logging filters information"));
+            vshError(ctl, "%s", _("Unable to get daemon logging filters information"));
             return false;
         }
 
@@ -1005,10 +1005,57 @@ static const vshCmdInfo info_daemon_log_outputs = {
                 "daemon."),
 };
 
+static const vshCmdOptDef opts_daemon_log_outputs[] = {
+    {.name = "outputs",
+     .type = VSH_OT_STRING,
+     .positional = true,
+     .help = N_("redefine the existing set of logging outputs"),
+     .allowEmpty = true
+    },
+    {.name = NULL}
+};
+
+static bool
+cmdDaemonLogOutputs(vshControl *ctl, const vshCmd *cmd)
+{
+    vshAdmControl *priv = ctl->privData;
+
+    if (vshCommandOptBool(cmd, "outputs")) {
+        const char *outputs = NULL;
+        if ((vshCommandOptString(ctl, cmd, "outputs", &outputs) < 0 ||
+             virAdmConnectSetLoggingOutputs(priv->conn, outputs, 0) < 0)) {
+            vshError(ctl, "%s", _("Unable to change daemon logging settings"));
+            return false;
+        }
+    } else {
+        g_autofree char *outputs = NULL;
+        if (virAdmConnectGetLoggingOutputs(priv->conn, &outputs, 0) < 0) {
+            vshError(ctl, "%s", _("Unable to get daemon logging outputs information"));
+            return false;
+        }
+
+        vshPrintExtra(ctl, " %-15s", _("Logging outputs: "));
+        vshPrint(ctl, "%s\n", NULLSTR_EMPTY(outputs));
+    }
+
+    return true;
+}
+
+
+/* --------------------------
+ * Command daemon-timeout
+ * --------------------------
+ */
+static const vshCmdInfo info_daemon_timeout = {
+    .help = N_("set the auto shutdown timeout of the daemon"),
+    .desc = N_("set the auto shutdown timeout of the daemon"),
+};
+
 static const vshCmdOptDef opts_daemon_timeout[] = {
     {.name = "timeout",
      .type = VSH_OT_INT,
      .required = true,
+     .positional = true,
      .help = N_("number of seconds the daemon will run without any active connection"),
     },
     {.name = NULL}
@@ -1030,51 +1077,6 @@ cmdDaemonTimeout(vshControl *ctl, const vshCmd *cmd)
 }
 
 
-/* --------------------------
- * Command daemon-timeout
- * --------------------------
- */
-static const vshCmdInfo info_daemon_timeout = {
-    .help = N_("set the auto shutdown timeout of the daemon"),
-    .desc = N_("set the auto shutdown timeout of the daemon"),
-};
-
-static const vshCmdOptDef opts_daemon_log_outputs[] = {
-    {.name = "outputs",
-     .type = VSH_OT_STRING,
-     .positional = true,
-     .help = N_("redefine the existing set of logging outputs"),
-     .allowEmpty = true
-    },
-    {.name = NULL}
-};
-
-static bool
-cmdDaemonLogOutputs(vshControl *ctl, const vshCmd *cmd)
-{
-    vshAdmControl *priv = ctl->privData;
-
-    if (vshCommandOptBool(cmd, "outputs")) {
-        const char *outputs = NULL;
-        if ((vshCommandOptString(ctl, cmd, "outputs", &outputs) < 0 ||
-             virAdmConnectSetLoggingOutputs(priv->conn, outputs, 0) < 0)) {
-            vshError(ctl, _("Unable to change daemon logging settings"));
-            return false;
-        }
-    } else {
-        g_autofree char *outputs = NULL;
-        if (virAdmConnectGetLoggingOutputs(priv->conn, &outputs, 0) < 0) {
-            vshError(ctl, _("Unable to get daemon logging outputs information"));
-            return false;
-        }
-
-        vshPrintExtra(ctl, " %-15s", _("Logging outputs: "));
-        vshPrint(ctl, "%s\n", NULLSTR_EMPTY(outputs));
-    }
-
-    return true;
-}
-
 static void *
 vshAdmConnectionHandler(vshControl *ctl)
 {
@@ -1269,8 +1271,8 @@ vshAdmParseArgv(vshControl *ctl, int argc, char **argv)
             break;
         case 'd':
             if (virStrToLong_i(optarg, NULL, 10, &debug) < 0) {
-                vshError(ctl, _("option %1$s takes a numeric argument"),
-                         longindex == -1 ? "-d" : "--debug");
+                const char *optStr = longindex == -1 ? "-d" : "--debug";
+                vshError(ctl, _("option %1$s takes a numeric argument"), optStr);
                 exit(EXIT_FAILURE);
             }
             if (debug < VSH_ERR_DEBUG || debug > VSH_ERR_ERROR)
@@ -1318,7 +1320,7 @@ vshAdmParseArgv(vshControl *ctl, int argc, char **argv)
                 vshError(ctl, _("unsupported option '%1$s'. See --help."), argv[optind - 1]);
             exit(EXIT_FAILURE);
         default:
-            vshError(ctl, _("unknown option"));
+            vshError(ctl, "%s", _("unknown option"));
             exit(EXIT_FAILURE);
         }
         longindex = -1;
@@ -1330,7 +1332,7 @@ vshAdmParseArgv(vshControl *ctl, int argc, char **argv)
         /* parse command */
         ctl->imode = false;
         if (argc - optind == 1) {
-            vshDebug(ctl, VSH_ERR_INFO, "commands: \"%s\"\n", argv[optind]);
+            vshDebug(ctl, VSH_ERR_INFO, "commands: \"%s\"", argv[optind]);
             return vshCommandStringParse(ctl, argv[optind], NULL);
         } else {
             return vshCommandArgvParse(ctl, argc - optind, argv + optind);
